from fabulaws.ubuntu.packages.base import AptMixin


class Fail2banMixin(AptMixin):
    """
    FabulAWS Ubuntu mixin that installs the fail2ban server for rejecting
    connections from IPs with a history of failed login attemps.
    """

    package_name = "fail2ban"
    fail2ban_packages = ["fail2ban"]



from fabulaws.ubuntu.instances.base import UbuntuInstance

__all__ = ["MicroLucidInstance", "SmallLucidInstance"]


class MicroLucidInstance(UbuntuInstance):
    """
    Sample Micro instance running Ubuntu Lucid on an EBS root filesystem.
    """

    ami = "ami-3e02f257"  # Lucid EBS
    instance_type = "t1.micro"
    key_prefix = "micro-ubuntu-"


class SmallLucidInstance(UbuntuInstance):
    """
    Sample Small instance running Ubuntu Lucid on on an instance-store root
    filesystem.
    """

    ami = "ami-7000f019"  # Lucid instance store
    instance_type = "m1.small"
    key_prefix = "small-ubuntu-"



import time

from fabric.api import settings, sudo
from fabric.contrib import files

from fabulaws.decorators import uses_fabric
from fabulaws.ubuntu.packages.base import AptMixin


class RabbitMqMixin(AptMixin):
    """
    FabulAWS Ubuntu mixin that configures RabbitMQ
    """

    package_name = "rabbitmq"
    rabbitmq_packages = ["rabbitmq-server"]
    rabbitmq_ulimit = 20000

    @uses_fabric
    def rabbitmq_service(self, cmd):
        return sudo("service rabbitmq-server {0}".format(cmd))

    def bind_app_directories(self, *args, **kwargs):
        tries = kwargs.pop("rabbitmq_tries", 10)
        sleep = kwargs.pop("rabbitmq_sleep", 2)
        # make sure we stop before proceeding in case we get moved to a secure directory
        self.rabbitmq_service("stop")
        super(RabbitMqMixin, self).bind_app_directories(*args, **kwargs)
        # try starting a number of times with warn_only=True, as rabbitmq
        # fails to restart occassionally for unknown reasons
        restarted = False
        for i in range(tries - 1):
            with settings(warn_only=True):
                result = self.rabbitmq_service("start")
                if result.return_code == 0:
                    restarted = True
                    break
                else:
                    time.sleep(sleep)
        # if we haven't succeeded yet, start again without warn_only=True
        # and let the failure be handled as usual by Fabric
        if not restarted:
            self.rabbitmq_service("start")

    @uses_fabric
    def rabbitmq_command(self, command):
        """Run a rabbitmqctl command."""

        sudo("rabbitmqctl %s" % command)

    def create_mq_user(self, username, password):
        """Create a rabbitmq user."""

        self.rabbitmq_command("add_user %s %s" % (username, password))

    def create_mq_vhost(self, name):
        """Create a rabbitmq vhost."""

        self.rabbitmq_command("add_vhost %s" % name)

    def set_mq_vhost_permissions(self, vhost, username, permissions='".*" ".*" ".*"'):
        """Set permssions for a user on a given vhost."""

        self.rabbitmq_command(
            "set_permissions -p %s %s %s" % (vhost, username, permissions)
        )

    @uses_fabric
    def rabbitmq_configure(self):
        files.append(
            "/etc/default/rabbitmq-server",
            "ulimit -n %s" % self.rabbitmq_ulimit,
            use_sudo=True,
        )
        self.rabbitmq_service("restart")

    def setup(self):
        """Redis mixin"""

        super(RabbitMqMixin, self).setup()
        self.rabbitmq_configure()


class RabbitMqOfficialMixin(RabbitMqMixin):
    """
    Installs RabbitMQ from the official upstream APT repository.
    """

    rabbitmq_aptrepo = (
        "http://www.rabbitmq.com/debian/",
        "testing",
        "main",
        "056E8E56",
    )
    rabbitmq_packages = ["rabbitmq-server"]



import datetime

from fabric.api import sudo
from fabric.contrib import files

from fabulaws.decorators import uses_fabric
from fabulaws.ubuntu.packages.base import AptMixin


class MemcachedMixin(AptMixin):
    """
    FabulAWS Ubuntu mixin that installs and configures memcached.
    """

    package_name = "memcached"
    memcached_packages = ["memcached"]
    memcached_ulimit = 10000
    memcached_bind = "127.0.0.1"  # set to '' to bind to all interfaces
    memcached_memory = "64"
    memcached_connections = "1024"
    memcached_conf = "/etc/memcached.conf"
    memcached_init_default = "/etc/default/memcached"
    memcached_conf_patterns = {
        "bind": ("-l", r"\S+"),
        "connections": ("-c", r"\w+"),
        "memory": ("-m", r"\w+"),
        "threads": ("-t", r"\w+"),
    }

    @uses_fabric
    def memcached_service(self, cmd):
        sudo("/etc/init.d/memcached {0}".format(cmd), pty=False)

    @uses_fabric
    def memcached_configure(self, **kwargs):
        for key, (opt, pat) in self.memcached_conf_patterns.items():
            if key in kwargs:
                val = kwargs[key]
            else:
                val = getattr(self, "memcached_{0}".format(key), None)
            if val is None:
                continue
            full_pat = r"^#?\s*{0}\s+{1}".format(opt, pat)
            full_val = "{0} {1}".format(opt, val)
            # we don't know if the config file contains the line or not, so
            # comment it out if it does and then add a new line with the date
            files.comment(self.memcached_conf, full_pat, use_sudo=True)
            if val != "":
                date = datetime.datetime.now().strftime("%Y-%m-%d")
                message = "\n# {0}; added by fabulaws {1}\n{2}" "".format(
                    key, date, full_val
                )
                files.append(self.memcached_conf, message, use_sudo=True)
        if self.memcached_ulimit is not None:
            ulimit = "ulimit -n {0}".format(self.memcached_ulimit)
            files.append(self.memcached_init_default, ulimit, use_sudo=True)
        self.memcached_service("restart")

    def setup(self):
        """memcached mixin"""

        super(MemcachedMixin, self).setup()
        self.memcached_configure()



from fabric.api import sudo

from fabulaws.decorators import uses_fabric
from fabulaws.ubuntu.packages.base import AptMixin


class PythonMixin(AptMixin):
    """
    FabulAWS Ubuntu mixin that installs Python
    """

    package_name = "python"
    python_ppa = None
    python_packages = ["python3", "python3-dev"]
    python_install_tools = True
    python_virtualenv_version = None  # install the latest version

    @uses_fabric
    def install_python_tools(self):
        """
        Installs the required Python tools from PyPI.
        """

        sudo("curl https://bootstrap.pypa.io/get-pip.py --output /tmp/get-pip.py")
        sudo("python3 /tmp/get-pip.py")
        version = ""
        if self.python_virtualenv_version:
            version = "==%s" % self.python_virtualenv_version
        sudo("pip install -U virtualenv%s" % version)

    def setup(self):
        """
        Hook into the FabulAWS setup() routine to install Python and related
        tools.
        """
        super(PythonMixin, self).setup()
        if self.python_install_tools:
            self.install_python_tools()



from fabric.api import cd, sudo
from fabric.contrib import files

from fabulaws.decorators import uses_fabric
from fabulaws.ubuntu.packages.base import AptMixin


class ShorewallMixin(AptMixin):
    """
    FabulAWS Ubuntu mixin that installs and configures the Shorewall firewall
    """

    package_name = "shorewall"
    shorewall_packages = ["shorewall"]
    shorewall_open_ports = ["SSH"]
    shorewall_custom = []
    shorewall_allow_icmp = False

    def _get_rules(self, ports):
        for p in ports:
            try:
                p = int(p)
                yield "ACCEPT net $FW tcp {0}".format(p)
            except ValueError:
                # looks like a macro, use this format instead
                yield "{0}/ACCEPT net $FW".format(p)

    @uses_fabric
    def _setup_firewall(self):
        """
        Configures and starts up a Shorewall firewall on the remote server.
        """
        rules = list(self._get_rules(self.shorewall_open_ports))
        rules.extend(self.shorewall_custom)
        with cd("/etc/shorewall"):
            sudo("rsync -a /usr/share/doc/shorewall/examples/one-interface/ .")
            if files.exists("shorewall.conf.gz"):
                sudo("gunzip -f shorewall.conf.gz")
            files.append("rules", "\n".join(rules), use_sudo=True)
            sudo('sed -i "s/STARTUP_ENABLED=No/STARTUP_ENABLED=Yes/" shorewall.conf')
        if self.shorewall_allow_icmp:
            files.sed(
                "/etc/shorewall/rules", r"Ping\(DROP\)", "Ping(ACCEPT)", use_sudo=True
            )
        sudo("shorewall start")

    def setup(self):
        """
        Hooks into the FabulAWS setup process to configure Shorewall on the
        server.
        """
        super(ShorewallMixin, self).setup()
        self._setup_firewall()
